!    This file is part of hybrid.
!    $Id: hybrid.f90,v 1.38 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****

! Calculate the hybridization according to
! J. C. Slater and G. F. Koster,
!   "Simplified LCAO Method for the Periodic Potential Problem"
!   Physical Review 94, 1498 (1954)
! and using Harrison's rules from
! Walter A. Harrison,
!    "Electronic Structure and the Properties of Solids",
!    Dover, 1989
!
! Contains:
! - "program" - read file, output, setup matrix etc.
! - subroutine usage()
!
program hybrid
  use general, only: r8, distance, degree
  use atomdata, only: ATOM_TYPE
  use slaterKoster, only: hyb_type
  use hybridization, only: showHybrid, printHyb, solvePDHyb
  use hybrid_read, only: opt_type, rot_type, band_type, readFile
  use tb, only: runTightBinding
  use hybrid_symm, only: symmetry
  implicit none

  intrinsic command_argument_count
  intrinsic get_command_argument
  character(*), parameter :: version = 'Hybrid, Version 2008-03-17'
  integer :: natoms
  type(atom_type),pointer :: tatoms(:)   ! Atom data
  real(r8),       pointer :: catoms(:,:) ! Atom coordinate
  type(hyb_type), pointer :: Vhyb(:,:)   ! user-defined hybridization
  integer :: i, j
  character(100)  :: str,filename
  logical         :: l_exists
  real(r8)        :: shift(3),scale(3),lattice(3),Emin,Emax
  integer         :: nkpts(3),numE
  type(rot_type)  :: rotmat
  type(opt_type)  :: option
  type(band_type) :: band
  character(10) dat,tim
  nullify(tatoms, catoms, Vhyb)

  str = 'atom.in' 
  if(command_argument_count() > 1) call usage(full=.true.)
  if(command_argument_count() == 1) then
    call get_command_argument(1, str)
    if(str == '-h' .or. str == '-help' .or. str =='--help') &
       call usage(full=.true.)
  end if
  
  filename = str
  inquire(file=filename,exist=l_exists)
  if(l_exists .eqv. .false.) then
    write(*,*) 'ERROR: readFile - cannot open file "', &
                trim(filename),'": Does not exist'
    call usage(full=.false.)
  end if
  print *, '------------ hybrid, version ',version,' -------------'
  call date_and_time(date=dat,time=tim)
  write(*,'("Date (YYYY-MM-DD) : ",A4,"-",A2,"-",A2)') &
        dat(1:4),dat(5:6), dat(7:8)
  write(*,'("Time (hh:mm:ss)   : ",A2,":",A2,":",A2)') &
        tim(1:2),tim(3:4), tim(5:6)

  print *, '--------- File: ',trim(filename),'-------------'
  call readFile(filename,natoms,catoms,tatoms,Vhyb,option,rotmat,shift,&
                scale,lattice,nkpts,Emin,Emax,numE,band)

  do i = 1,natoms
     print '(i2,a,a8,a5,a4,3f15.8)', i,': ',trim(tatoms(i)%name), &
           ' ('//trim(tatoms(i)%elem)//')',' at ',catoms(:,i)
  end do

  ! Rotate coordinates
  if(.not. rotmat%identity .or. &
     abs(shift(1))+abs(shift(2))+abs(shift(3))>tiny(shift) .or. &
     scale(1) /= 1.0_r8 .or. scale(2) /= 1.0_r8 .or. scale(3) /=1.0_r8) then
     write(*,'(/,a,/)') 'Shifted, scaled and rotated coordinates:'
     write(*,'(a,/,3(a,f9.5),/)') 'Shifted coordinates (before rotating) by ',&
          'x = ', shift(1), ', y = ', shift(2), ', z = ', shift(3)
     write(*,'(a)') 'Euler angles:'
     if (rotmat%y /= 0.0_r8) then
       write(*,'(a,f9.5,a,f9.6,a)') 'z1 = phi   = ',rotmat%z1/degree, ' deg = ',rotmat%z1,' rad'
       write(*,'(a,f9.5,a,f9.6,a)') 'y  = theta = ',rotmat%y /degree, ' deg = ',rotmat%y, ' rad'
       write(*,'(a,f9.5,a,f9.6,a)') 'z2 = psi   = ',rotmat%z2/degree, ' deg = ',rotmat%z2,' rad'
     else
       write(*,'(a,2(f9.5,a))') 'z1 = phi   = ',rotmat%z1/degree, ' deg = ',rotmat%z1,' rad'
       write(*,'(a,2(f9.5,a))') 'x  = theta = ',rotmat%x /degree, ' deg = ',rotmat%x, ' rad'
       write(*,'(a,2(f9.5,a))') 'z2 = psi   = ',rotmat%z2/degree, ' deg = ',rotmat%z2,' rad'
     end if
     write(*,'(a,3(/,3f9.5))') 'Rotation matrix: ',transpose(rotmat%mat) ! Fortran has row vectors
     write(*,'(a,3f9.5)') '   (1,0,0) -> ',matmul(rotmat%mat,(/ 1.0_r8, 0.0_r8, 0.0_r8 /))
     write(*,'(a,3f9.5)') '   (0,1,0) -> ',matmul(rotmat%mat,(/ 0.0_r8, 1.0_r8, 0.0_r8 /))
     write(*,'(a,3f9.5)') '   (0,0,1) -> ',matmul(rotmat%mat,(/ 0.0_r8, 0.0_r8, 1.0_r8 /))
     write(*,'(/,3(a,f9.5),/)') 'Scaled by a1 = ',scale(1),', a2 = ',scale(2),' a3 = ',scale(3)
     write(*,'(/,a)') 'New coordinates:'

     do i = 1,natoms
        catoms(:,i) = catoms(:,i) + shift(:)
        catoms(:,i) = matmul(rotmat%mat,catoms(:,i))*scale(:)
        print '(i2,a,a8,a5,a4,3f15.8)', i,': ',trim(tatoms(i)%name), &
              ' ('//trim(tatoms(i)%elem)//')',' at ',catoms(:,i)
     end do
  end if
  if(option%tbDOS .or. option%tbBand) then
     write(*,'(/,3(a,f9.5))') 'LATTICE: a1 = ',lattice(1),&
           ', a2 = ',lattice(2),', a3 = ',lattice(3)
     write(*,'(/,3(a,i5),/)') 'Number of k points: k1 = ',nkpts(1),&
           ', k2 = ',nkpts(2),', k3 = ',nkpts(3)
     write(*,'(/,a)') 'Tight-binding on-site energies:'
     do i = 1,natoms
       print '(i2,3a,3(a,f9.5))', i,': ',trim(tatoms(i)%name), &
            ' ('//trim(tatoms(i)%elem)//')',' Es = ',tatoms(i)%Es,&
            ' Ep = ',tatoms(i)%Ep,' Ed = ',tatoms(i)%Ed
     end do
     if(option%tbDOS) then
       write(*,'(/,2(a,f9.5),a,i0/)') 'DOS energy range: Emin = ',Emin,&
             ';  Emax = ',Emax,', steps: ',numE
     end if
  end if

  do i = 1,natoms
    do j = i+1,natoms
      write(str,*) 'User defined (',trim(tatoms(i)%elem),', ',i,')--(', trim(tatoms(j)%elem),', ',j,'): '
      call printHyb(str, &
!                 -1,&
                tatoms(i)%vsubshell, &
                tatoms(j)%vsubshell,Vhyb(i,j))
      if(0.0_r8 == distance(catoms(:,i),catoms(:,j))) then
        write(*,*) 'ERROR: Atoms ',i,' and ',j,' have the same coordinate!'
        stop
      end if
    end do
  end do
  print *, ''

  if(option%symm) call symmetry(natoms,catoms, tatoms)

  if(option%showHyb)  call showHybrid(natoms,tatoms,catoms,Vhyb)

  if(option%doPDHyb) then
     call solvePDHyb(natoms,catoms,tatoms,Vhyb,option)
  end if

  if(option%tbDOS .or. option%tbBand) then
     if(filename(len_trim(filename)-2:len_trim(filename)) == '.in') then
        str = filename(1:len_trim(filename)-3)
     else
        str = filename
     end if
     call runTightBinding(natoms,catoms,tatoms,Vhyb,lattice,nkpts, &
              DOS=option%tbDOS,band=option%tbBand,filebase=str, &
              Emin=Emin,Emax=Emax,numE=numE,bands=band)
  end if

  if(associated(tatoms)) deallocate(tatoms)
  if(associated(catoms)) deallocate(catoms)

contains

  subroutine usage(full)
    implicit none
    logical :: full
    print *, 'USAGE: hybrid [--help] [filename]'
    print *, '  Reads "atom.in" or the given filename and calculates the hybridization'
    print *, '--help   show usage and example input file'
    print *, 'Version: ',version
    print *,''
    if(full) then
      print *,'--------- Example atom.in -------------------------'
      print *,'! Calculate the hybridization ("!" starts comment lines)'
!      print *,'&opt showHyb=f doPDHyb=true tbDOS=false tbBand=f maxiter=20 /'
      print *,'&opt showHyb=f doPDHyb=true tbDOS=false maxiter=20 /'
      print *,'! maxiter: Number of iterations for symmetrizing the p-d matrix'
      print *,''
      print *,' 3  ! Thee atoms (empty lines are ignored also)'
      print *,' 0.0 0.0 0.0 Co  ! Cobalt ion at  (0,0,0)'
      print *,' 1.0 0.0 0.0 O   ! Oxygen ion at  (1,0,0)'
      print *,'-1.0 0.0 0.0 O   ! Oxygen ion at (-1,0,0)'
      print *,''
      print *,'! Optionally, specify the hybridization'
      print *,'! either absolutely in eV (pds, pdp etc.) or relative to'
      print *,'! Harrison''s rules (factor), e.g. fpds, fpdp etc.'
      print *,'!&hyb atom1=1, atom2=2 pds=-2.0 pdp=3.0 /'
      print *,'!&hyb atom1=1, atom2=3 fpds=1.15 fpdp=1.15 /'
      print *,'!&shift x=0 y=10.3 z=-0.4 / ! shift; applies before rotation'
      print *,'! Rotate coordinates (Euler angles):'
      print *,'! rotate 20 degree around z1, then 10 around x'
      print *,'!&rot z1=20 x=10 z2=0 /'
      print *,'!&rot z1=1.345 y=4.5 z2=0 degrees=false /'
      print *,'!Scale coordinates (after shifting)'
      print *,'!&scale a1=6.1 a2=6.3 a3=10.3 /'
      print *,'!Rotate this atom into [0,0,1] direction (use after &shift); the'
      print *,'!second atom is rotated into the [1,0,z] direction.'
      print *,'!&autorot atom=1 atom2=2/'
      print *,''
!      print *,'!------ tight binding --------'
!      print *,'!&tb a1=6.1 a2=6.3 a3=10.3 nk1=10 nk2=10 nk3=20 Emin=-10 Emax=10 numE=100/'
!      print *,'!&tbsite atom=1 Es=10 Ep=3 Ed=33.3 /'
    end if
    stop
  end subroutine
end program hybrid
