!    This file is part of hybrid.
!    $Id: hybrid_potential.f90,v 1.24 2007-07-11 15:45:29 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****

! This module is not optimized for speed. However, on all non-ancient computers
! it should run in a fraction of a second; therefore, there is no need to
! optimize the routines. (Readability and correctness of the source code
! is much moreimportant than any millisecond gained!)

module potential
  use general, only : r8
  implicit none
!  integer, parameter, private :: rT = selected_real_kind(17,200) ! kind=10 or 16
  integer, parameter, private :: rT = r8

! Contains:
! - subroutine expandSphH(V,B,isCmplx) - expand potential in spherical harmonics
! - subroutine hybPD(m,Hin,Hout) - diagonalize p-d hybridization
! - subroutine rotH(n,H)         - make upper-right/lower-bottom submatrix symmetric
! - subroutine QRDecomp(m,n,A,Q) - QR decomposition
! - subroutine SVD(m,n,A,U)      - Single-value decomposition (returns only U)

contains

! Expand a potential in spherical harmonics
! Note: This assumes d orbitals
! See: Maurits W. Haverkort, Ph.D. thesis (Koeln, 2005)
! OAI: arxiv.org:cond-mat/0505214
! Arguments: V(5,5) matrix containing the potential
!   rows: different ligands, columns: transition metal
! Returns:  B(0:4,-4:4) the coefficients
!           isCmplx (optional) is real or complex
! Assumed index order: xy, yz, zx, x2-y2, 3z2-r2
subroutine expandSphH(V,B,isCmplx)
  use general, only: r8
  implicit none
  real(r8),    intent(in)  :: V(5,5)
  complex(r8), intent(out) :: B(0:4,-4:4)
  logical,     intent(out),optional :: isCmplx
  complex(r8) :: I
  logical :: l_isCmplx
  I = cmplx(0.0_r8, 1.0_r8, r8)
  l_isCmplx = .false.
  B = cmplx(0.0_r8, 0.0_r8, r8)

  B(0, 0) = (V(1,1)+V(2,2)+V(3,3)+V(4,4)+V(5,5))/5.0_r8
  B(2, 0) = 0.5_r8*(-2.0_r8*V(4,4) - 2.0_r8*V(1,1) + V(3,3) + V(2,2) &
                    + 2.0_r8*V(5,5))
  B(2, 1) = 0.5_r8*(I*sqrt(6.0_r8)*V(1,3) - sqrt(6.0_r8)*V(1,2) &
                    -sqrt(6.0_r8)*V(3,4) - sqrt(2.0_r8)*V(3,5)  &
                    -I*sqrt(6.0_r8)*V(2,4) + I*sqrt(2.0_r8)*V(2,5))
  B(2,-1) = conjg(B(2,1))*(-1.0_r8) ! (-1)^1
  B(2, 2) = 0.25_r8*(-4.0_r8*sqrt(2.0_r8)*V(4,5) + 4.0_r8*I*sqrt(2.0_r8)*V(1,5)&
                    + sqrt(6.0_r8)*V(3,3)        - sqrt(6.0_r8)*V(2,2)         &
                    - 2.0_r8*I*sqrt(6.0_r8)*V(2,3))
  B(2,-2) = conjg(B(2, 2)) ! * (-1)^2
  B(4, 0) = 0.3_r8*(V(4,4)+V(1,1)-4.0_r8*V(3,3)-4.0_r8*V(2,2)+6.0_r8*V(5,5))
  B(4, 1) = 0.3_r8*(-I*sqrt(5.0_r8)*V(1,3)  + sqrt(5.0_r8)*V(1,2)         &
                    + sqrt(5.0_r8)*V(3,4)   - 2.0_r8*sqrt(15.0_r8)*V(3,5) &
                    + I*sqrt(5.0_r8)*V(2,4) + 2.0_r8*sqrt(15.0_r8)*I*V(2,5)) 
  B(4,-1) = conjg(B(4, 1)) * (-1.0_r8) ! (-1)^1
  B(4, 2) = 0.3_r8*(sqrt(30.0_r8)*V(4,5) - I*sqrt(30.0_r8)*V(1,5) &
                  + sqrt(10.0_r8)*V(3,3) - sqrt(10.0_r8)*V(2,2)   &
                  - 2.0_r8*I*sqrt(10.0_r8)*V(2,3))
  B(4,-2) = conjg(B(4, 2)) ! * (-1)^2
  B(4, 3) = 3.0_r8*sqrt(35.0_r8)/10.0_r8*(I*V(1,3)+V(1,2)-V(3,4)+I*V(2,4))
  B(4,-3) = conjg(B(4, 3)) * (-1.0_r8) ! * (-1)^3
  B(4, 4) = 3.0_r8*sqrt(70.0_r8)/20.0_r8*(V(4,4)-V(1,1)-2.0_r8*I*V(1,4))
  B(4,-4) = conjg(B(4, 4)) ! * (-1)^4

  if(present(isCmplx)) then
    isCmplx = .false.
    if( abs(V(1,3)) + abs(V(1,4)) + abs(V(1,5))                         &
      + abs(V(2,3)) + abs(V(2,4)) + abs(V(2,5)) > 1e3_r8*tiny(V(1,1)) ) &
        isCmplx = .true.
  end if
  ! Used to be: 1e3_r8*tiny(V(1,1)
  if(abs(V(2,1)-V(1,2)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring        V(Ld_yz,xy) = ', V(2,1)
      write(*,*) '                      |V(Ld_yz,xy) - V(yz,Ld_xy)| = ',abs(V(2,1)-V(1,2))
  end if
  if(abs(V(3,1)-V(1,3)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring        V(Ld_zx,xy) = ', V(3,1)
      write(*,*) '                      |V(Ld_zx,xy) - V(zx,Ld_xy)| = ',abs(V(3,1)-V(1,3))
  end if
  if(abs(V(3,2)-V(2,3)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring        V(Ld_zx,yz) = ', V(3,2)
      write(*,*) '                      |V(Ld_zx,yz) - V(zx,Ld_yz)| = ',abs(V(3,2)-V(2,3))
  end if
  if(abs(V(4,1)-V(1,4)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring     V(Ld_x2-y2,xy) = ', V(4,1)
      write(*,*) '                |V(Ld_x2-y2,xy) - V(x2-y2,Ld_xy)| = ',abs(V(4,1)-V(1,4))
  end if
  if(abs(V(4,2)-V(2,4)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring     V(Ld_x2-y2,yz) = ', V(4,2)
      write(*,*) '                |V(Ld_x2-y2,yz) - V(x2-y2,Ld_yz)| = ',abs(V(4,2)-V(2,4))
  end if
  if(abs(V(4,3)-V(3,4)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring     V(Ld_x2-y2,zx) = ', V(4,3)
      write(*,*) '                |V(Ld_x2-y2,zx) - V(x2-y2,Ld_zx)| = ',abs(V(4,3)-V(3,4))
  end if
  if(abs(V(5,1)-V(1,5)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring    V(Ld_3z2-r2,xy) = ', V(5,1)
      write(*,*) '              |V(Ld_3z2-r2,xy) - V(3z2-r2,Ld_xy)| = ',abs(V(5,1)-V(1,5))
  end if
  if(abs(V(5,2)-V(2,5)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring    V(Ld_3z2-r2,yz) = ', V(5,2)
      write(*,*) '              |V(Ld_3z2-r2,yz) - V(3z2-r2,Ld_yz)| = ',abs(V(5,2)-V(2,5))
  end if
  if(abs(V(5,3)-V(3,5)) > 1e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring    V(Ld_3z2-r2,zx) = ', V(5,3)
      write(*,*) '              |V(Ld_3z2-r2,zx) - V(3z2-r2,Ld_zx)| = ',abs(V(5,3)-V(3,5))
  end if
  if(abs(V(5,4)-V(4,5)) > 2e-7_r8 ) then
      write(*,*) 'WARNING: expandSphH - ignoring V(Ld_3z2-r2,x2-y2) = ', V(5,4)
      write(*,*) '        |V(Ld_3z2-r2,x2-y2) - V(3z2-r2,Ld_x2-y2)| = ',abs(V(5,4)-V(4,5))
  end if
end subroutine expandSphH

! Numerically diagonizes a (m,5) matrix, which represents the
! hybridization of mod(m,5) ligands with one transition metal
!
!
subroutine hybPD(m,Hin,Hout,maxit)
  use general, only: r8
  implicit none
  integer, parameter           :: n = 5 ! Warning, check algorithm if you change this
  integer, intent(in)          :: m
  integer, intent(in),optional :: maxit ! Maximal number of iterations
  real(r8), intent(in)         :: Hin(m,n+m)
  real(r8), intent(out)        :: Hout(n,2*n)
  integer :: m2, i, j, k, maxiter
  logical :: ok
  real(r8), allocatable,dimension(:,:) :: A,Q,T,H,U

  maxiter = 20
  if(present(maxit)) maxiter = maxit

  ! A = Hin -> p-d mixing Hamiltonian
  ! Q -> Q of QR decomposition
  ! T -> transformation matrix (unitary)
  ! H -> full Hamiltonian, needed for Hout' = T.H.T^(transp)

  m2 = m
  if(m < n) then
    m2 = n ! our QRdecomp assumes m >= n
    write(*,*) 'ERROR: Currently, at least two ligands are needed'
    stop
  end if
 

  allocate(A(m2,n),Q(m2,m2),H(5+m2,5+m2),T(5+m2,5+m2))

  if(m < n) then
     A(1:m,:)    = Hin(:,1:5)
     A(m+1:m2,:) = 0.0_r8
  else
     A = Hin(:,1:5)
  end if

  ! Without this trick, the dimension k of Q(k <= min(n,m),m)
  ! of the QR decomposition becomes too small
  ! (e.g. 4 instead of 5 for 3 atoms at (0,0,0), (0,0,z), (0,0,-z) )
!  A = A + 1.0d-20

  ! Save Hamiltonian + make big Hamiltonian
  H = 0.0_r8
  do i = 1,5+m2
    H(i,i) = 1d0
  end do
  H(5+1:5+m,1:5) = A
  H(1:5,5+1:5+m) = transpose(A)
  H(5+1:5+m,5+1:5+m) = Hin(:,6:)

!  print *, '------------- Full hamiltonian ------------------'
!  do i = 1,5
!    print '(i2,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:)
!  end do
!  print *, '------------------------------------------------------------------------------'
!  do i = 6,5+m2
!    print '(i2,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:)
!  end do

  call QRDecomp(m2,n,A,Q)

  T = 0.0_r8
  do i = 1,5+m2
    T(i,i) = 1d0
  end do
  T(5+1:5+5,5+1:5+m) = Q(1:5,1:m)

!  print *, '------------- Full Unitary matrix ------------------'
!  do i = 1,5
!    print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, T(i,1:5),' | ',T(i,6:10),' | ', T(i,11:)
!  end do
!  print *, '------------------------------------------------------------------------------'
!  do i = 6,10
!    print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, T(i,1:5),' | ',T(i,6:10),' | ', T(i,11:)
!  end do
!  print *, '------------------------------------------------------------------------------'
!  do i = 11,5+m2
!    print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, T(i,1:5),' | ',T(i,6:10),' | ', T(i,11:)
!  end do

  if(m > 5) then
    allocate(U(m,m))
    call SVD(m2,n,Hin,U)
    T(5+5+1:5+m,5+1:5+m) = transpose(U(:,6:))
    deallocate(U)
  end if

!  print *, '------------- Full Unitary matrix ------------------'
!  do i = 1,5
!    print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, T(i,1:5),' | ',T(i,6:10),' | ', T(i,11:)
!  end do
!  print *, '------------------------------------------------------------------------------'
!  do i = 6,10
!    print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, T(i,1:5),' | ',T(i,6:10),' | ', T(i,11:)
!  end do
!  print *, '------------------------------------------------------------------------------'
!  do i = 11,5+m2
!    print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, T(i,1:5),' | ',T(i,6:10),' | ', T(i,11:)
!  end do

  H = matmul(T,matmul(H,transpose(T)))


!  print *, '------------- Full hamiltonian ------------------'
! do i = 1,5
!   print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:10),' | ', H(i,11:)
! end do
! print *, '------------------------------------------------------------------------------'
! do i = 6,10
!   print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:10),' | ', H(i,11:)
! end do
! print *, '------------------------------------------------------------------------------'
! do i = 11,5+m2
!   print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:10),' | ', H(i,11:)
! end do

  ! Arbitray choice: make diagonal of Hout positive;
  ! rotH takes care that this is also true for the changes
  T = 0.0_r8
  do i = 1,5+m2
    T(i,i) = 1.0_r8
  end do
  do i = 1,5
     if(H(5+i,i) < 0.0_r8) T(5+i,5+i) = -1.0_r8
  end do
  H = matmul(T,matmul(H,transpose(T)))
  deallocate(T,A)
  allocate(A(10,10),T(10,10))
  A = H(1:10, 1:10) 
  T = 0.0_rT
  do i = 1, 10
     T(i,i) = 1.0_rT
  end do
  if(maxiter > 0) then
    !Symmetrize Hout
    i = 0
    do
      ok = .true.
      ! Check symmetry status
      do j = 1,5
        do k = j+1,5
          if(abs(A(5+j,k) - A(5+k,j)) > 1e-5_r8) ok = .false.
        end do
      end do
      if(ok) then
        print *, 'Symmetrizing: Ok after ', i, ' iterations'
        exit
      end if
      if((.not. ok) .and. (i >= maxiter)) then
        print *, 'WARNING: Symmetrizing aborted after ',i,' iterations'
        exit
      end if
      call findRotT(10,H(1:10,1:10),T)
      A = matmul(transpose(T),matmul(H(1:10,1:10),T))      
      i = i + 1
    end do
  end if

! print *, '------------- Full hamiltonian ------------------'
! do i = 1,5
!   print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:10),' | ', H(i,11:)
! end do
! print *, '------------------------------------------------------------------------------'
! do i = 6,10
!   print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:10),' | ', H(i,11:)
! end do
! print *, '------------------------------------------------------------------------------'
! do i = 11,5+m2
!   print '(i2,5f5.2,a,5f5.2,a,100f5.2)', i, H(i,1:5),' | ',H(i,6:10),' | ', H(i,11:)
! end do
!
  Hout = A(5+1:5+5, 1:10)
end subroutine hybPD


! Rotate H such that the upper right/lower bottom
! blocks are symmetric submatrices; assumes that
! H itself is Hermitian
subroutine findRotT(n,H,T)
  use general, only: r8, M_PI, degree
  implicit none
  integer,intent(in) :: n
  real(r8),intent(in) :: H(n,n)
  real(rT),intent(inout) :: T(n,n)
  real(r8) :: Htmp(n,n), Ttmp(n,n)
  integer :: i,j,k
  real(r8) :: ang

  Htmp = matmul(T,matmul(H,transpose(T)))
  do i = 1, 5 
    do j = 1+1, 5
      if(abs(Htmp(i+5,j)-Htmp(i,j+5)) < 1e-10_r8) cycle ! nothing to do
      print *,'(i,j) = ', i,j
      Ttmp = 0.0_r8
      do k = 1, n
        Ttmp(k,k) = 1.0_r8
      end do
      if(abs(Htmp(i+5,j)) < 1e-10_r8) then
!        print *, '(1) a=',H(i+5,i),', b = ',H(j+5,i),' d = ',H(j+5,j)
        ang = acos( (Htmp(i+5,i) + Htmp(j+5,j))  &
                   / sqrt(Htmp(j,i+5)**2 + (Htmp(i+5,i) + Htmp(j+5,j))**2) )
      elseif(abs(Htmp(i,j+5)) < 1e-10_r8) then
!        print *, '(2) a=',Htmp(i+5,i),', c = ',Htmp(i+5,j),' d = ',Htmp(j+5,j)
        ang = sign(1.0_r8,Htmp(i+5,j))*          &
              acos( (Htmp(i+5,i) + Htmp(j+5,j))  &
                   / sqrt(Htmp(i+5,j)**2 + (Htmp(i+5,i) + Htmp(j+5,j))**2) )
      else
!          print *, '(3) a=',H(i+5,i),', b = ',H(i+5,j),', c = ',H(i+5,j),' d = ',H(j+5,j)
          !ang = acos( a+d /sqrt( [b-c]**2 +[a+d]**2 )*sign[b-c] )  ! modulo sign inside/outside acos
          ang = acos(-(Htmp(i+5,i)+Htmp(j+5,j))  &
                     / sqrt( (Htmp(i+5,j)-Htmp(j+5,i))**2 + (Htmp(i+5,i)+Htmp(j+5,j))**2 ))
      end if
!      print *, 'acos ',ang, ' = ',ang*180.0_r8/M_PI
      Ttmp(i+5,i+5) =  cos(ang)
      Ttmp(j+5,j+5) =  cos(ang)
      Ttmp(i+5,j+5) =  sin(ang)
      Ttmp(j+5,i+5) = -sin(ang)
      if(cos(ang)*Htmp(i+5,i)+sin(ang)*Htmp(j+5,i) < 0.0_r8) &
        Ttmp(i+5,i+5) = Ttmp(i+5,i+5)*(-1.0_r8)
      if(cos(ang)*Htmp(i+5,i)-sin(ang)*Htmp(i+5,j) < 0.0_r8) &
        Ttmp(j+5,j+5) = Ttmp(j+5,j+5)*(-1.0_r8)
      
!      print *, '---- t2 -----------'
!      do k = 1,10
!        print '(i2,5f5.2,a,5f5.2,a,100f5.2)', k, T(1:5,k),'|',T(6:,k)
!      end do
      T = matmul(transpose(T),Ttmp)
      Htmp = matmul(transpose(T),matmul(H,T))
     print *, '------------- findRotT: Full hamiltonian ------------------'
     do k = 1,5
       print '(i2,5f6.2,a,5f6.2,a,100f6.2)', k, Htmp(k,1:5),' | ',Htmp(k,6:10)
     end do
    end do
  end do
end subroutine findRotT

! Does QR decomposition of vector A(m,n)
! Returns: A contains "R" in upper triangle (n,n) ! see assumtion
!          Q contains "Q" (n,m) ! see assumtion
! This routine assumes m >= n
! See also G.H. Golub and C. F. van Loan, Matrix Computation (Hopkins, 1996).
! And http://www.nag.co.uk/numeric/FL/manual/pdf/F08/f08_intro.pdf
subroutine QRDecomp(m,n,A,Q)
  implicit none
  interface ! http://www.netlib.org/lapack, see also http://www.nag.co.uk/numeric/fl/manual/pdf/F08/f08aef.pdf
    ! DGEQRF - computes a QR factorization of a real M-by-N matrix A: A = Q * R.
    subroutine dgeqrf(m,n,A,lda,Tau,Work,lwork,info)
      integer          :: m, n, lda, lwork, info
      double precision :: A(lda,n),Tau(*),Work(lwork)
    end subroutine dgeqrf
    ! DORGQR - generate an M-by-N real matrix Q with orthonormal columns
    ! -> for m >= n
    subroutine dorgqr(m,n,k,A,lda,Tau,Work,lwork,info)
      integer          :: m,n,k,lda,lwork,info
      double precision :: A(lda,n), tau(*),work(lwork)
    end subroutine dorgqr
  end interface
  integer, intent(in)             :: m, n
  double precision, intent(inout) :: A(m,n)
  double precision, intent(out)   :: Q(m,m)

  integer :: info, lwork, lda
  double precision, allocatable :: tau(:), work(:), H(:,:)
  lwork = n
  lda   = m

  if(m < n .or. n < 1) then
    write(*,*) 'ERROR: QRDecomp: Subroutine assumes 0 < n <= m, got n=',n,', m=',m
    stop
  end if

  allocate(tau(n),work(lwork),H(m,m)) 
  call dgeqrf(m, n, A, lda, tau, Work, lwork, info)
  if(info /= 0) then
    write(*,*) 'ERROR: QRDecomp - DGEQRF returned 0 /= info = ',info
    stop
  end if

  Q(:,1:5) = A

  call dorgqr(m, n, n, Q, LDA, TAU, Work, lwork, info)
  if(info /= 0) then
    write(*,*) 'ERROR: QRDecomp - DORGQR returned 0 /= info = ',info
    stop
  end if
  Q = transpose(Q)
deallocate(tau,work,H)
end subroutine QRDecomp

! Calculates the Single-value decomposition
! See G.H. Golub and C. F. van Loan, Matrix Computation (Hopkins, 1996).
! Decomposes A into U(Sigma)V^T, where (Sigma) is an (m,n) matrix, which is
! zero for the min(m,n) diagonal elements, U is an (m,m) orthogonal matrix and
! V an (n,n) orthogonal matrix.
!
! WARNING: For simplicity, only U is calculated; if you need more, you need
! to modify the dgesvd call!
! We assume further m >= n as otherwise the null space calculation does not make sense
subroutine SVD(m,n,A,U)
  implicit none
  interface ! From LAPACK, see also http://www.nag.co.uk/numeric/fl/manual/pdf/F08/f08kbf.pdf
    subroutine dgesvd(jobu, jobvt, m, n, A, lda, S, U, ldu, VT, ldvt, Work, lwork, info)
      character        :: jobu, jobvt
      integer          :: m, n, lda, ldu, ldvt, lwork, info
      double precision :: A(lda,n), S(*), U(ldu,*), VT(ldvt,n), work(lwork)
    end subroutine dgesvd
  end interface

  integer,          intent(in)  :: m,n
  double precision, intent(in)  :: A(m,n)
  double precision, intent(out) :: U(m,m) !  S(:), VT(n,n)
  double precision :: VT(1) ! as jobvt='N', VT is not refered
  double precision :: S(m) ! S(max(m,n)), assuming m > n
  integer          :: lwork, info
  double precision,allocatable :: Work(:)
  lwork = 5*m ! >= max(1, 3min(m,n)+max(m,n), 5*min(m,n))
  allocate(Work(lwork))

  if(n > m .or. n < 1) then
    write(*,*) 'ERROR: hybrid_potential - SVD: this implementation assumes 0 < n < m'
    write(*,*) '  However, m = ',m,' and n = ',n,' were passed.'
    stop
  end if

  ! jobvt='N'-> don't calculate VT, jobz = 'A' -> calculate all U
  call dgesvd('A', 'N', m, n, A, m, S, U, m, VT, 1, Work, lwork, info)
  if(info /= 0) then
    write(*,*) 'ERROR: SVD - DGESVD returned 0 /= info = ',info
    stop
  end if
  deallocate(Work)
end subroutine SVD
end module potential
