!    This file is part of hybrid.
!    $Id: hybrid_slater.f90,v 1.9 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****

! Calculate the hybridization according to
! J. C. Slater and G. F. Koster,
!   "Simplified LCAO Method for the Periodic Potential Problem"
!   Physical Review 94, 1498 (1954)
! and using Harrison's rules from
! Walter A. Harrison,
!    "Electronic Structure and the Properties of Solids",
!    Dover, 1989
!
! Contains:
! - type hyb_type (to save all the pds etc. potentials)
! - hbar2, nu_sss etc. private constants
! - function labelPt(pt) - transform 1 into 'x', 2 into 'y' etc.
! - function labelDt(dt) - transform 1 into 'xy', 2 into 'yz' etc.
! - function HarrisonRd(Z) - gets Harrison's r_d for d-elements
! - function Vsss(d), Vsps(d) etc. - determine ss(sigma) etc.
! - subroutine Ess(x, fVsss), Esp(),Epp etc. determine Slater-
!   Koster parameters (only pre-factors of Vsss, Vsps etc.)
!
module slaterKoster
  use general, only: r8
  implicit none

  type hyb_type
    real(r8) :: Vsss, Vsps, Vpps, Vppp, Vsds, Vpds, Vpdp, &
                Vdds, Vddp, Vddd, &
                fVsss, fVsps, fVpps, fVppp, fVsds, fVpds, fVpdp, &
                fVdds, fVddp, fVddd
  end type hyb_type

  ! See Harrison (1989)
  real(r8),parameter :: hbar2m =  7.62_r8 ! hbar^2/m in eV-Ang^2
  real(r8),parameter :: nu_sss = -1.40_r8
  real(r8),parameter :: nu_sps =  1.84_r8
  real(r8),parameter :: nu_pps =  3.24_r8
  real(r8),parameter :: nu_ppp = -0.81_r8
  real(r8),parameter :: nu_sds = -3.16_r8
  real(r8),parameter :: nu_pds = -2.95_r8
  real(r8),parameter :: nu_pdp =  1.36_r8
  real(r8),parameter :: nu_dds =-16.20_r8
  real(r8),parameter :: nu_ddp =  8.75_r8
  real(r8),parameter :: nu_ddd =  0.00_r8
contains

  function labelPt(pt)
    implicit none
    integer, intent(in) :: pt
    character(1) :: labelPt
    select case(pt)
      case(1); labelPt = 'x'
      case(2); labelPt = 'y'
      case(3); labelPt = 'z'
      case default
        write(*,'(a,i0)') 'ERROR: labelPt - Unknown p-orbital type: ',pt
        stop 'ERROR: labelPt - unknown p-orbital type'
    end select
  end function


  function labelDt(dt)
    implicit none
    integer, intent(in) :: dt
    character(6) :: labelDt
    select case(dt)
      case(1); labelDt = 'xy'
      case(2); labelDt = 'yz'
      case(3); labelDt = 'zx'
      case(4); labelDt = 'x2-y2'
      case(5); labelDt = '3z2-r2'
      case default
        write(*,'(a,i0)') 'ERROR: labelDt - Unknown d-orbital type: ',dt
        stop 'ERROR: labelDt - unknown d-orbital type'
    end select
  end function

  ! Return Harrison's r_d for transition metals
  ! Parameter: Z atom number
  ! Return: r_d in Aangstroem
  ! See: W. A. Harrison (1989)
  function HarrisonRd(Z)
    use general, only: r8
    implicit none
    integer, intent(in)   :: Z
    real(r8)              :: HarrisonRd
    real(r8), parameter, dimension(21:29) ::  &
     rD3 = (/ 1.24_r8, 1.08_r8, 0.98_r8, 0.90_r8, 0.86_r8, &
              ! Sc      Ti       V        Cr       Mn
              ! 21      22       23       24       25
              0.80_r8, 0.76_r8, 0.71_r8, 0.67_r8 &
              ! Fe      Co       Ni       Cu
           /) ! 26      27       28       29
    real(r8), parameter, dimension(39:47) ::  &
     rD4 = (/ 1.58_r8, 1.41_r8, 1.28_r8, 1.20_r8, 1.11_r8, &
              ! Y       Zr       Nb       Mo       Tc
              ! 39      40       41       42       43
              1.05_r8, 0.99_r8, 0.94_r8, 0.89_r8 &
             ! Ru      Rh       Pd       Ag
           /) ! 44      45       46       47
    real(r8), parameter, dimension(71:79) ::  &
     rD5 = (/ 1.58_r8, 1.44_r8, 1.34_r8, 1.27_r8, 1.20_r8, &
              ! Lu      Hf       Ta       W        Re
              ! 71      72       73       74       75
              1.13_r8, 1.08_r8, 1.04_r8, 1.01_r8 &
              ! Os      Ir       Pt       Au
           /) ! 76      77       78       79

    if(21 <= Z .and. Z <= 29 ) then
       HarrisonRd = rD3(Z)
    elseif(39 <= Z .and. Z <= 47) then
       HarrisonRd = rD4(Z)
    elseif(71 <= Z .and. Z <= 79) then
       HarrisonRd = rD5(Z)
    else
      write(*,'(a)')   'ERROR: Cannot return HarrisonRd for non-d elements'
      write(*,'(a,i0)')'       Passed: Z = ',Z
      stop 'ERROR: HarrisonRd - non-d element'
    end if
  end function HarrisonRd

  ! Same as HarrisonRd, but returns text
  subroutine HarrisonRdA(Z, str)
     implicit none
     integer, intent(in)          :: Z
     character(len=*),intent(out) :: str

     integer :: i

     i = int(HarrisonRd(Z)*100.0_r8)
     write(str,'(a,i0,a)') '(',i,'/100)'
  end subroutine HarrisonRdA

  ! Determine pd(sigma) etc. according to Harrison's rule

  function Vsss(d)
     implicit none
     real(r8),intent(in) :: d
     real(r8)            :: Vsss
     Vsss = nu_sss*hbar2m/d**2
  end function

  function Vsps(d)
     implicit none
     real(r8),intent(in) :: d
     real(r8)            :: Vsps
     Vsps = nu_sps*hbar2m/d**2
  end function

  function Vpps(d)
     implicit none
     real(r8),intent(in) :: d
     real(r8)            :: Vpps
     Vpps = nu_pps*hbar2m/d**2
  end function

  function Vppp(d)
     implicit none
     real(r8),intent(in) :: d
     real(r8)            :: Vppp
     Vppp = nu_ppp*hbar2m/d**2
  end function

  function Vsds(d,rd)
     implicit none
     real(r8),intent(in) :: d,rd
     real(r8)            :: Vsds
     Vsds = nu_sds*hbar2m*(rd**(1.5_r8))/(d**(3.5_r8))
  end function

  function Vpds(d,rd)
     implicit none
     real(r8),intent(in) :: d,rd
     real(r8)            :: Vpds
     Vpds = nu_pds*hbar2m*(rd**(1.5_r8))/(d**(3.5_r8))
  end function

  function Vpdp(d,rd)
     implicit none
     real(r8),intent(in) :: d,rd
     real(r8)            :: Vpdp
     Vpdp = nu_pdp*hbar2m*(rd**(1.5_r8))/(d**(3.5_r8))
  end function

  function Vdds(d,rd)
     implicit none
     real(r8),intent(in) :: d,rd
     real(r8)            :: Vdds
     Vdds = nu_dds*hbar2m*rd**3/d**5
  end function

  function Vddp(d,rd)
     implicit none
     real(r8),intent(in) :: d,rd
     real(r8)            :: Vddp
     Vddp = nu_ddp*hbar2m*rd**3/d**5
  end function

  function Vddd(d,rd)
     implicit none
     real(r8),intent(in) :: d,rd
     real(r8)            :: Vddd
     Vddd = nu_ddd*hbar2m*rd**3/d**5
  end function


  ! For the following hybridization energies, see
  ! Slater and Koster (1954)

  ! Trivial function!
  !! s-s hybridization
  !! E_(s,s) = Vss(sigma)
  !! Parameters: x(3) = (l, m, n) - coordinate
  !! Returns:    fVsss - factor for V_ss(sigma)
  !subroutine Ess(x, fVsss)
  !   implicit none
  !   real(r8), intent(in) :: x(3)
  !   real(r8), intent(out):: fVsss
  !   fVsss = 1.0_r8
  !end subroutine Ess

  ! s-p hybridization
  ! Parameters: x(3) = (l, m, n)  - coordinate
  !             pt       - type of p orbital: 1 (x), 2 (y) or 3 (z)
  ! Returns:    fVsps - factor for V_sp(sigma)
  subroutine Esp(x, pt, fVsps)
    use general, only: r8
    implicit none
    real(r8), intent(in) :: x(3)
    integer,  intent(in) :: pt
    real(r8), intent(out):: fVsps

    if(pt < 1 .or. pt > 3) then
      write(*,'(a)')    'ERROR: Esp - wrong p-orbital type'
      write(*,'(a,i0)') '       allowed: 1,2,3; received: pt = ',pt
      stop
    end if

    fVsps = x(pt)
  end subroutine Esp

  ! s-p hybridization [STRING VERSION]
  ! Parameters: x(3) = (l, m, n)  - coordinate
  !             pt       - type of p orbital: 1 (x), 2 (y) or 3 (z)
  ! Returns:    fVsps - factor for V_sp(sigma)
  subroutine EspA(x, pt, fVsps)
    implicit none
    character(*), intent(in) :: x(3)
    integer, intent(in)      :: pt
    character(*), intent(out):: fVsps

    if(pt < 1 .or. pt > 3) then
      write(*,'(a)')    'ERROR: EspA - wrong p-orbital type'
      write(*,'(a,i0)') '       allowed: 1,2,3; received: pt = ',pt
      stop
    end if

    fVsps = x(pt)
  end subroutine EspA

  ! p-p hybridization
  ! Parameters: x(3) = (l, m, n)  - coordinate
  !             pt1, pt2 - type of p orbital: 1 (x), 2 (y) or 3 (z)
  ! Returns:    fVpps - factor for V_sp(sigma)
  !             fVppp - factor for V_sp(pi)
  subroutine Epp(x, pt1, pt2, fVpps, fVppp)
     use general, only: r8
     implicit none
     real(r8), intent(in) :: x(3)
     integer,  intent(in) :: pt1, pt2
     real(r8), intent(out):: fVpps, fVppp

     if(pt1 < 1 .or. pt1 > 3 .or. pt2 < 1 .or. pt2 > 3) then
       write(*,'(a)')       'ERROR: Epp - wrong p-orbital type'
       write(*,'(a)')       '       allowed: 1,2,3'
       write(*,'(2(a,i0))') '       received: pt1 = ',pt1,' pt2 = ',pt2
       stop
     end if

     if(pt1 == pt2) then
        fVpps = x(pt1)**2
        fVppp = (1.0_r8-x(pt1)**2)
     else
        fVpps =  x(pt1)*x(pt2)
        fVppp = -x(pt1)*x(pt2)
     end if
  end subroutine Epp


  ! p-p hybridization [STRING VERSION]
  ! Parameters: x(3) = (l, m, n)  - coordinate
  !             pt1, pt2 - type of p orbital: 1 (x), 2 (y) or 3 (z)
  ! Returns:    fVpps - factor for V_sp(sigma)
  !             fVppp - factor for V_sp(pi)
  subroutine EppA(x, pt1, pt2, fVpps, fVppp)
     implicit none
     character(*), intent(in) :: x(3)
     integer, intent(in)      :: pt1, pt2
     character(*), intent(out):: fVpps, fVppp

     if(pt1 < 1 .or. pt1 > 3 .or. pt2 < 1 .or. pt2 > 3) then
       write(*,'(a)')       'ERROR: Epp - wrong p-orbital type'
       write(*,'(a)')       '       allowed: 1,2,3'
       write(*,'(2(a,i0))') '       received: pt1 = ',pt1,' pt2 = ',pt2
       stop
     end if

     if(pt1 == pt2) then
        if(x(pt1) == '0') then
           fVpps = '0'
           fVppp = '1'
        else
           fVpps = '('//x(pt1)//')^2'
           fVppp = '(1-('//x(pt1)//')^2)'
        end if
     else
        if(x(pt1) == '0' .or. x(pt2) == '0') then
          fVpps = '0'
          fVppp = '0'
        else
          fVpps =  '('//x(pt1)//')*('//x(pt2)//')'
          fVppp = '-('//x(pt1)//')*('//x(pt2)//')'
        end if
     end if
  end subroutine EppA

  ! s-d hybridization
  ! Parameters: l, m, n  - coordinate
  !             dt - type of d orbital:
  !              xy(1), yz(2), zx(3),x2-y2(4),3z2-r2(5)
  ! Returns:    fVsds - factor for V_sd(sigma)
  subroutine Esd(l, m, n, dt, fVsds)
     use general, only: r8
     implicit none
     real(r8), intent(in) :: l, m, n
     integer,  intent(in) :: dt
     real(r8), intent(out):: fVsds

     if(dt < 1 .or. dt > 5) then
       write(*,'(a)')    'ERROR: Esd - wrong d-orbital type'
       write(*,'(a)')    '       allowed: 1,2,3,4,5'
       write(*,'(a,i0)') '       received: dt = ',dt
       stop
     end if

     if(dt == 1) then ! s,xy
       fVsds = sqrt(3.0_r8)*l*m
     elseif(dt == 2) then ! s,yz
       fVsds = sqrt(3.0_r8)*m*n
     elseif(dt == 3) then ! s,zx
       fVsds = sqrt(3.0_r8)*n*l
     elseif(dt == 4) then ! s,x2-y2
       fVsds = 0.5_r8*sqrt(3.0_r8)*(l**2-m**2)
     elseif(dt == 5) then ! s,3z2-r2
       fVsds = (n**2 - 0.5_r8*(l**2 + m**2))
     end if
  end subroutine Esd

  ! p-d hybridization
  ! Parameters: l, m, n  - coordinate
  !             pt, dt - type of p and d orbital:
  !              x(1), y(2), z(3)  
  !              xy(1), yz(2), zx(3),x2-y2(4),3z2-r2(5)
  ! Returns:    fVpds - factor for V_pd(sigma)
  !             fVpdp - factor for V_pd(pi)
  subroutine Epd(l, m, n, pt, dt, fVpds, fVpdp)
     use general, only: r8
     implicit none
     real(r8), intent(in) :: l, m, n
     integer,  intent(in) :: pt, dt
     real(r8), intent(out):: fVpds, fVpdp

     if(pt < 1 .or. pt > 3) then
       write(*,'(a)')    'ERROR: Epd - wrong p-orbital type'
       write(*,'(a)')    '       allowed: 1,2,3'
       write(*,'(a,i0)') '       received: pt = ',pt
       stop
     end if
     if(dt < 1 .or. dt > 5) then
       write(*,'(a)')    'ERROR: Epd - wrong d-orbital type'
       write(*,'(a)')    '       allowed: 1,2,3,4,5'
       write(*,'(a,i0)') '       received: dt = ',dt
       stop
     end if

     if(dt == 1) then ! p,xy
       if(pt == 1) then ! x,xy
         fVpds = sqrt(3.0_r8)*l**2*m
         fVpdp = m*(1.0_r8-2.0_r8*l**2)
       elseif(pt == 2) then ! y,xy
         fVpds = sqrt(3.0_r8)*m**2*l
         fVpdp = l*(1.0_r8-2.0_r8*m**2)
       elseif(pt == 3) then ! z,xy
         fVpds = sqrt(3.0_r8)*l*m*n
         fVpdp = -2.0_r8*l*m*n
       end if
     elseif(dt == 2) then ! p,yz
       if(pt == 1) then ! x,yz
         fVpds = sqrt(3.0_r8)*l*m*n
         fVpdp = -2.0_r8*l*m*n
       elseif(pt == 2) then ! y,yz
         fVpds = sqrt(3.0_r8)*m**2*n
         fVpdp = n*(1.0_r8-2.0_r8*m**2)
       elseif(pt == 3) then ! z,yz
         fVpds = sqrt(3.0_r8)*n**2*m
         fVpdp = m*(1.0_r8-2.0_r8*n**2)
       end if
     elseif(dt == 3) then ! p,xz
       if(pt == 1) then ! x,zx
         fVpds = sqrt(3.0_r8)*l**2*n
         fVpdp = n*(1.0_r8-2.0_r8*l**2)
       elseif(pt == 2) then ! y,zx
         fVpds = sqrt(3.0_r8)*l*m*n
         fVpdp = -2.0_r8*l*m*n
       elseif(pt == 3) then ! z,zx
         fVpds = sqrt(3.0_r8)*n**2*l
         fVpdp = l*(1.0_r8-2.0_r8*n**2)
       end if
     elseif(dt == 4) then ! p,x2-y2
       if(pt == 1) then ! x,x2-y2
         fVpds = 0.5_r8*sqrt(3.0_r8)*l*(l**2-m**2)
         fVpdp = l*(1.0_r8-l**2+m**2)
       elseif(pt == 2) then ! y,x2-y2
         fVpds =  0.5_r8*sqrt(3.0_r8)*m*(l**2-m**2)
         fVpdp = -m*(1.0_r8+l**2-m**2)
       elseif(pt == 3) then ! z,x2-y2
         fVpds =  0.5_r8*sqrt(3.0_r8)*n*(l**2-m**2)
         fVpdp = -n*(l**2-m**2)
       end if
     elseif(dt == 5) then ! p,3z2-r2
       if(pt == 1) then ! x,3z2-r2
         fVpds =  l*(n**2-0.5_r8*(l**2+m**2))
         fVpdp = -sqrt(3.0_r8)*l*n**2
       elseif(pt == 2) then ! y,3z2-r2
         fVpds =  m*(n**2-0.5_r8*(l**2+m**2))
         fVpdp = -sqrt(3.0_r8)*m*n**2
       elseif(pt == 3) then ! z,3z2-r2
         fVpds =  n*(n**2-0.5_r8*(l**2+m**2))
         fVpdp =  sqrt(3.0_r8)*n*(l**2+m**2)
       end if
     end if
  end subroutine Epd

  ! FIXME/TODO: Finish this one
  ! d-d hybridization
  ! Parameters: l, m, n  - coordinate
  !             dt1, dt2 - type of d orbitals:
  !              xy(1), yz(2), zx(3),x2-y2(4),3z2-r2(5)
  ! Returns:    fVdds - factor for V_dd(sigma)
  !             fVddp - factor for V_dd(pi)
  !             fVddd - factor for V_dd(delta)
  subroutine Edd(l, m, n, d1, d2, fVdds, fVddp, fVddd)
     use general, only: r8
     implicit none
     real(r8), intent(in) :: l, m, n
     integer,  intent(in) :: d1, d2
     real(r8), intent(out):: fVdds, fVddp, fVddd
     integer              :: dt1, dt2

     if(d1 < 1 .or. d1 > 5 .or. d2 < 1 .or. d2 > 5) then
       write(*,'(a)')    'ERROR: Epd - wrong d-orbital type'
       write(*,'(a)')    '       allowed: 1,2,3,4,5'
       write(*,'(a,i0)') '       received: dt1 = ',d1,' dt2=',d2
       stop
     end if

     if(d1 < d2) then
       dt1 = d1
       dt2 = d2
     else
       dt1 = d2
       dt2 = d1
     end if

     print *, 'dt:',dt1,dt2,'  d:',d1,d2

     if(dt1 == 1) then ! xy
       if(dt2 == 1) then ! xy
         fVdds = 3.0_r8*l**2*m**2
         fVddp = l**2 + m**2 - 4.0_r8*l**2*m**2
         fVddd = n**2 + l**2*m**2
       elseif(dt2 == 2) then ! yz
         fVdds = 3.0_r8*l*m**2*n
         fVddp = l*n*(1.0_r8 - 4.0_r8*m**2)
         fVddd = l*n*(m**2 - 1.0_r8)
       elseif(dt2 == 3) then ! zx
         fVdds = 3.0_r8*l**2*m*n
         fVddp = m*n*(1.0_r8 - 4.0_r8*l**2)
         fVddd = m*n*(l**2-1.0_r8)
       elseif(dt2 == 4) then ! x2-y2
         fVdds = 1.5_r8*l*m*(l**2 - m**2)
         fVddp = 2.0_r8*l*m*(m**2 - l**2)
         fVddd = 0.5_r8*l*m*(l**2 - m**2)
       elseif(dt2 == 5) then ! 3z2-r2
         fVdds = sqrt(3.0_r8)*l*m*(n**2 - 0.5_r8*(l**2 + m**2))
         fVddp = sqrt(3.0_r8)*2.0_r8*l*m*n**2
         fVddd = 0.5_r8*sqrt(3.0_r8)*l*m*(1.0_r8 + n**2)
       end if
     elseif(dt1 == 2) then ! yz
       if(dt2 == 1) then ! xy
         fVdds = 3.0_r8*m**2*n*l
         fVddp = n*l*(1.0_r8 - 4.0_r8*m**2)
         fVddd = n*l*(m**2-1.0_r8)
       elseif(dt2 == 2) then ! yz
         fVdds = 3.0_r8*m**2*n**2
         fVddp = m**2 + n**2 - 4.0_r8*m**2*n**2
         fVddd = l**2 + m**2*n**2
       elseif(dt2 == 3) then ! zx
         fVdds = 3.0_r8*m*n**2*l
         fVddp = m*l*(1.0_r8 - 4.0_r8*n**2)
         fVddd = m*l*(n**2 - 1.0_r8)
       elseif(dt2 == 4) then ! x2-y2
         fVdds = 1.5_r8*m*n*(l**2 - m**2)
         fVddp = m*n*(1.0_r8 + 2.0_r8*(l**2 - m**2))
         fVddd = m*n*(1.0_r8 - 0.5_r8*(l**2 - m**2))
       elseif(dt2 == 5) then ! 3z2-r2
         fVdds = sqrt(3.0_r8)*m*n*(n**2 - 0.5_r8*(l**2 + m**2))
         fVddp = sqrt(3.0_r8)*m*n*(l**2 + m**2 - n**2)
         fVddd = 0.5_r8*sqrt(3.0_r8)*m*n*(l**2 + m**2)
       end if
     elseif(dt1 == 3) then ! zx
       if(dt2 == 1) then ! xy
         fVdds = 3.0_r8*n*l**2*m
         fVddp = n*m*(1.0_r8-4.0_r8*l**2)
         fVddd = n*m*(l**2-1.0_r8)
       elseif(dt2 == 2) then ! yz
         fVdds = 3.0_r8*n**2*l*m
         fVddp = m*n*(1.0_r8-4.0_r8*n**2)
         fVddd = l*m*(n**2-1.0_r8)
       elseif(dt2 == 3) then ! zx
         fVdds = 3.0_r8*n**2*l**2
         fVddp = n**2+l**2-4.0_r8*n**2*l**2
         fVddd = m**2+n**2*l**2
       elseif(dt2 == 4) then ! x2-y2
         fVdds = 1.5_r8*n*l*(l**2 - m**2)
         fVddp = n*l*(1.0_r8 - 2.0_r8*(l**2 - m**2))
         fVddd = n*l*(1.0_r8 - 0.5_r8*(l**2 - m**2))
       elseif(dt2 == 5) then ! 3z2-r2
         fVdds = sqrt(3.0_r8)*l*n*(n**2 - 0.5_r8*(l**2 + m**2))
         fVddp = sqrt(3.0_r8)*l*n*(l**2 + m**2)
         fVddd = 0.5_r8*sqrt(3.0_r8)*l*n*(l**2 + m**2)
       end if
     elseif(dt1 == 4) then ! x2-y2
       ! since dt1 < dt2, we need no other cases
       if(dt2 == 4) then ! x2-y2
         fVdds = 0.75_r8*(l**2 - m**2)**2
         fVddp = l**2 + m**2 - (l**2 - m**2)**2
         fVddd = n**2 + 0.25_r8*(l**2 - m**2)**2
       elseif(dt2 == 5) then ! 3z2-r2
         fVdds = 0.5_r8*sqrt(3.0_r8)*(l**2 - m**2)*(n**2 - 0.5_r8*(l**2 + m**2))
         fVddp = sqrt(3.0_r8)*n**2*(m**2 - l**2)
         fVddd = 0.25_r8*sqrt(3.0_r8)*(1.0_r8 + n**2)*(l**2 - m**2)
       end if
     elseif(dt1 == 5) then ! 3z2-r2
       ! since dt1 < dt2, we need no other cases
       ! if(dt2 == 5) then ! 3z2-r2
         fVdds = (n**2 - 0.5_r8*(l**2 + m**2))**2
         fVddp = 3.0_r8*n**2*(l**2 + m**2)
         fVddd = 0.75_r8*(l**2 + m**2)**2
       ! end if
     end if
  end subroutine Edd

  ! Hybridization with f (and g) orbitals, see
  ! R. R. Sharma, "General expressions for reducing the Slater-Koster linear
  !   combination of atomic orbitals integrals to the two-center approximation",
  !  Phys. Rev. B 19, 2813 (1979)
  !  http://link.aps.org/abstract/PRB/v19/p2813
  subroutine Esf(); end subroutine
  subroutine Epf(); end subroutine
  subroutine Edf(); end subroutine
  subroutine Eff(); end subroutine
end module slaterKoster
